@@ -1,76 +1,64 @@
+import inspect
 import json
+import os
 import threading
+from enum import IntEnum, unique
 from time import strftime
 
 global_lock = threading.Lock()
 
 
-def _merge(msg) -> str:
-    if isinstance(msg, list):
-        msg = f'[{" ".join([str(x).strip() for x in msg])}]'
-    elif isinstance(msg, tuple):
-        msg = f'({" ".join([str(x).strip() for x in msg])})'
-    elif isinstance(msg, dict):
+def _merge(msg, frame: bool = True) -> str:
+    if isinstance(msg, dict):
         msg = f'\n{json.dumps(msg, indent=2)}'
+    elif isinstance(msg, tuple):
+        msg = " ".join([str(x).strip() for x in msg])
+        if frame:
+            msg = f'({msg})'
     else:
-        msg = f'[{str(msg)}]'
+        if isinstance(msg, str):
+            pass
+        elif hasattr(msg, '__iter__'):
+            msg = " ".join([str(x).strip() for x in msg])
+        else:
+            msg = str(msg)
+        if frame:
+            msg = f'[{msg}]'
 
     return msg
 
 
-class LoggerLevel:
+@unique
+class LoggerLevel(IntEnum):
     TRACE = 1
     DEBUG = 2
     INFO = 3
     SILENT = 4
 
-    group = [TRACE, DEBUG, INFO, SILENT]
 
-    def __init__(self, level):
-        if level not in self.group:
-            raise ValueError(f'log level error: {level}')
-        self.value = level
+def get_call_location_info(func):
+    def wrapper(*args, **kwargs):
+        cf = inspect.currentframe()
 
-    def __lt__(self, other):
-        if not isinstance(other, LoggerLevel):
-            return False
-        return self.value < other.value
+        line_no = cf.f_back.f_lineno
+        file_name = cf.f_back.f_code.co_filename
+        file_name = os.path.basename(file_name)
+        kwargs.update({
+            'line_no': line_no,
+            'file_name': file_name})
+        return func(*args, **kwargs)
 
-    def __le__(self, other):
-        if not isinstance(other, LoggerLevel):
-            return False
-        return self.value <= other.value
-
-    def __eq__(self, other):
-        if not isinstance(other, LoggerLevel):
-            return False
-        return self.value == other.value
-
-    def __ne__(self, other):
-        if not isinstance(other, LoggerLevel):
-            return False
-        return self.value != other.value
-
-    def __gt__(self, other):
-        if not isinstance(other, LoggerLevel):
-            return False
-        return self.value > other.value
-
-    def __ge__(self, other):
-        if not isinstance(other, LoggerLevel):
-            return False
-        return self.value >= other.value
+    return wrapper
 
 
 class Logger:
-    TRACE = LoggerLevel(LoggerLevel.TRACE)
-    DEBUG = LoggerLevel(LoggerLevel.DEBUG)
-    INFO = LoggerLevel(LoggerLevel.INFO)
-    SILENT = LoggerLevel(LoggerLevel.SILENT)
+    TRACE = LoggerLevel.TRACE
+    DEBUG = LoggerLevel.DEBUG
+    INFO = LoggerLevel.INFO
+    SILENT = LoggerLevel.SILENT
 
-    log_level_list = [TRACE, DEBUG, INFO, SILENT]
-
-    def __init__(self, prefix, level, handler=None, skip_repeat: bool = False, timestamp: str = "%Y%m%d %H:%M:%S"):
+    def __init__(self, prefix, level: LoggerLevel = INFO, handler=None, skip_repeat: bool = False,
+                 timestamp: str = "%Y%m%d %H:%M:%S"):
         self.prefix = prefix
         if not self.prefix:
             self.prefix = ''
@@ -79,8 +67,6 @@ class Logger:
 
         if not isinstance(level, LoggerLevel):
             raise TypeError(f'Error log level type: {type(level)}')
-        if level not in self.log_level_list:
-            raise ValueError('Log level error')
 
         self.level = level
 
@@ -96,16 +82,17 @@ class Logger:
         self.timestamp = timestamp
 
     def info(self, *msg):
-        self.log(Logger.INFO, *msg)
+        self._log(Logger.INFO, *msg)
 
-    def debug(self, *msg):
-        self.log(Logger.DEBUG, *msg)
+    @get_call_location_info
+    def debug(self, *msg, **kwargs):
+        self._log(Logger.DEBUG, *msg, **kwargs)
 
-    def trace(self, *msg):
-        self.log(Logger.TRACE, *msg)
-
-    def log(self, log_level: LoggerLevel, *msg):
+    @get_call_location_info
+    def trace(self, *msg, **kwargs):
+        self._log(Logger.TRACE, *msg, **kwargs)
 
+    def _log(self, log_level: LoggerLevel, *msg, line_no=None, file_name=None):
         if self.skip_repeat:
             if self.last_msg == msg:
                 return
@@ -114,7 +101,7 @@ class Logger:
         if len(msg) == 0:
             return
 
-        if log_level not in self.log_level_list:
+        if not isinstance(log_level, LoggerLevel):
             raise ValueError('Log level error')
 
         if self.level > log_level:
@@ -123,17 +110,15 @@ class Logger:
         if len(msg) == 0:
             return
 
-        des = str(msg[0])
-        msg = msg[1:]
+        des = _merge(msg[0], frame=False)
 
-        msg = [f' {_merge(x)}' for x in msg]
+        msg = [f' {_merge(x)}' for x in msg[1:]]
         msg.insert(0, des)
 
-        timestamp = ''
-        if self.timestamp:
-            timestamp = f'[{strftime(self.timestamp)}]'
+        timestamp = f'[{strftime(self.timestamp)}]' if self.timestamp else ''
+        location = f'[{file_name} {line_no}]' if line_no is not None else ''
 
-        total_message = f'{timestamp}{self.prefix} {"".join(msg)}'.strip()
+        total_message = f'{timestamp}{self.prefix}{location} {"".join(msg)}'.strip()
 
         with global_lock:
             if self.handler:
